package space.uranos.mdsp

import okhttp3.OkHttpClient
import okhttp3.Request
import java.io.File
import kotlin.time.measureTime

class DataDownloader(private val dir: File) {
    private val client = OkHttpClient()

    fun start() {
        FILES.forEach { (name, url) ->
            val file = dir.resolve(name)
            file.parentFile.mkdirs()

            if (file.exists()) println("$name already exists, skipping download")
            else {
                val request = Request.Builder()
                    .get()
                    .url(url)
                    .build()

                val time = measureTime {
                    client.newCall(request).execute().use {
                        if (it.isSuccessful) {
                            file.createNewFile()
                            it.body!!.byteStream().copyTo(file.outputStream())
                        } else throw DownloadException(name)
                    }
                }

                println("Downloaded $name in ${time.inSeconds}s")
            }
        }
    }

    class DownloadException(fileName: String) : Exception("Downloading $fileName failed")

    companion object {
        val FILES = mapOf(
            "minecraft_server.jar" to "https://launcher.mojang.com/v1/objects/35139deedbd5182953cf1caa23835da59ca3d7cd/server.jar",
            "blocks.json" to PRISMARINE_BASE_URL + "1.16.2/blocks.json",
            "biomes.json" to PRISMARINE_BASE_URL + "1.16.2/biomes.json",
            "entities.json" to PRISMARINE_BASE_URL + "1.16.2/entities.json",
            "blockCollisionShapes.json" to PRISMARINE_BASE_URL + "1.16.1/blockCollisionShapes.json"
        )
    }
}
