package space.uranos.mdsp

import com.jsoniter.JsonIterator
import org.gradle.api.Plugin
import org.gradle.api.Project
import space.uranos.mdsp.generator.*

class MinecraftDataSourcesPlugin : Plugin<Project> {
    override fun apply(project: Project) {
        val workingDir = project.file("buildSrcTemp")
        val outputDir = project
            .project(":uranos-api")
            .projectDir
            .resolve("src/main/generatedKotlin")

        val sourcesDir = project
            .project(":uranos-api")
            .projectDir
            .resolve("src/main/kotlin")

        val downloadTask = project.task("downloadMinecraftData") {
            it.group = TASK_GROUP

            it.doLast {
                DataDownloader(workingDir).start()
            }
        }

        val runDataGeneratorsTask = project.task("runDataGenerators") {
            it.group = TASK_GROUP
            it.dependsOn(downloadTask)

            it.doLast {
                DataGeneratorRunner(workingDir, outputDir).generate()
            }
        }

        project.task("generate") {
            it.group = TASK_GROUP
            it.dependsOn(downloadTask)
            it.dependsOn(runDataGeneratorsTask)

            it.doLast {
                outputDir.deleteRecursively()
                outputDir.mkdirs()

                val registries = JsonIterator.deserialize(workingDir.resolve("generated/reports/registries.json").readText())

                BlocksAndMaterialGenerator(workingDir, outputDir, sourcesDir).generate()
                TagsGenerator(workingDir, outputDir).generate()
                EntitiesGenerator(workingDir, outputDir, sourcesDir).generate()
                FluidIDMapGenerator(workingDir, outputDir, registries).generate()
                ItemTypeEnumGenerator(workingDir, outputDir, registries).generate()
            }
        }
    }

    companion object {
        const val TASK_GROUP = "minecraft data"
    }
}
