package space.uranos.mdsp.generator

import com.squareup.kotlinpoet.FileSpec
import com.squareup.kotlinpoet.ParameterizedTypeName.Companion.parameterizedBy
import com.squareup.kotlinpoet.PropertySpec
import space.uranos.mdsp.JsonAny
import java.io.File

class FluidIDMapGenerator(
    private val workingDir: File,
    private val outputDir: File,
    private val registries: JsonAny
) {
    fun generate() {
        val fluids = registries.get("minecraft:fluid").get("entries").asMap().map { it.key to it.value.get("protocol_id").toInt() }.toMap()
        val args = fluids.flatMap { listOf(it.key, it.value) }.toTypedArray()

        val property = PropertySpec.builder("NUMERIC_FLUID_IDS_BY_ID", Map::class.parameterizedBy(String::class, Int::class))
            .initializer("mapOf(\n${fluids.values.indices.joinToString(",\n") { "%S to %L" }}\n)", *args)
            .build()

        FileSpec.builder(WORLD_PACKAGE, "NumericFluidIDsByID")
            .addProperty(property)
            .build()
            .writeTo(outputDir)
    }
}
