package space.uranos.mdsp.generator

import com.google.common.base.CaseFormat
import com.squareup.kotlinpoet.*
import com.squareup.kotlinpoet.ParameterizedTypeName.Companion.parameterizedBy
import space.uranos.mdsp.JsonAny
import space.uranos.mdsp.util.ConstructorPropertiesHelper
import java.io.File

class ItemTypeEnumGenerator(
    private val workingDir: File,
    private val outputDir: File,
    private val registries: JsonAny
) {
    fun generate() {
        val cph = ConstructorPropertiesHelper()

        val builder = TypeSpec.enumBuilder("ItemType")
            .primaryConstructor(
                FunSpec.constructorBuilder()
                    .addParameters(
                        listOf(
                            cph.create("numericID", Int::class),
                            cph.create("id", String::class)
                        )
                    )
                    .build()
            )
            .addProperties(cph.getProperties())

        val items = registries.get("minecraft:item").get("entries").asMap()
            .map { it.key to it.value.get("protocol_id").toInt() }.sortedBy { it.second }

        for (item in items) {
            val id = item.first
            val name = CaseFormat.LOWER_UNDERSCORE.to(CaseFormat.UPPER_UNDERSCORE, id.removePrefix("minecraft:"))

            builder.addEnumConstant(
                name,
                TypeSpec.anonymousClassBuilder()
                    .addSuperclassConstructorParameter("%L", item.second)
                    .addSuperclassConstructorParameter("%S", id)
                    .build()
            )
        }

        builder.addType(
            TypeSpec.companionObjectBuilder()
                .addProperty(
                    PropertySpec.builder(
                        "byID",
                        Map::class.asTypeName().parameterizedBy(String::class.asTypeName(), ITEM_TYPE_TYPE)
                    )
                        .initializer("values().map { it.id to it }.toMap()")
                        .build()
                )
                .build()
        )

        FileSpec.builder(ITEM_PACKAGE, "ItemType")
            .addType(builder.build())
            .build()
            .writeTo(outputDir)
    }
}
