package space.uranos.mdsp.generator

import com.jsoniter.JsonIterator
import com.squareup.kotlinpoet.FileSpec
import com.squareup.kotlinpoet.KModifier
import com.squareup.kotlinpoet.ParameterizedTypeName.Companion.parameterizedBy
import com.squareup.kotlinpoet.PropertySpec
import com.squareup.kotlinpoet.asTypeName
import java.io.File

class TagsGenerator(workingDir: File, private val outputDir: File) {
    companion object {
        val TAG_TYPE_TYPES_BY_DIR_NAME = mapOf(
            "blocks" to TAG_TYPE_TYPE.nestedClass("BLOCKS"),
            "entity_types" to TAG_TYPE_TYPE.nestedClass("ENTITY_TYPES"),
            "fluids" to TAG_TYPE_TYPE.nestedClass("FLUIDS"),
            "items" to TAG_TYPE_TYPE.nestedClass("ITEMS")
        )
    }

    private val tagsDir = workingDir.resolve("generated/data/minecraft/tags")

    fun generate() {
        val entries: List<Pair<String, Array<Any>>> = TAG_TYPE_TYPES_BY_DIR_NAME.flatMap { (dirName, tagTypeType) ->
            tagsDir.resolve(dirName).listFiles()!!.map { file ->
                val name = "minecraft:" + file.nameWithoutExtension
                val json = JsonIterator.deserialize(file.readText())

                val values = json.get("values").asList().map { it.toString() }

                Pair(
                    "%T(%S, %T, listOf(\n${values.joinToString(",\n") { "%S".prependIndent("    ") }}\n))",
                    arrayOf(TAG_TYPE, name, tagTypeType, *values.toTypedArray())
                )
            }
        }

        val property = PropertySpec.builder("MINECRAFT_INTERNAL_TAGS", List::class.asTypeName().parameterizedBy(TAG_TYPE))
            .initializer(
                "listOf(\n${entries.joinToString(",\n") { it.first.prependIndent("    ") }}\n)",
                *entries.flatMap { it.second.toList() }.toTypedArray()
            )
            .addModifiers(KModifier.INTERNAL)
            .build()

        FileSpec.builder(TAG_PACKAGE, "MinecraftInternalTags")
            .addProperty(property)
            .build()
            .writeTo(outputDir)
    }
}
