package space.uranos.testplugin

import space.uranos.Uranos
import space.uranos.chat.ChatColor
import space.uranos.chat.TextComponent
import space.uranos.entity.MinecartEntity
import space.uranos.entity.Position
import space.uranos.net.ServerListInfo
import space.uranos.net.event.ServerListInfoRequestEvent
import space.uranos.net.event.SessionAfterLoginEvent
import space.uranos.player.GameMode
import space.uranos.plugin.Plugin
import space.uranos.testplugin.anvil.AnvilWorld
import space.uranos.util.RGBColor
import space.uranos.util.runInServerThread
import space.uranos.util.secondsToTicks
import space.uranos.world.*
import space.uranos.world.block.GreenWoolBlock
import space.uranos.world.block.RedWoolBlock

class TestPlugin : Plugin("Test", "1.0.0") {
    fun enableOptifineFix() {
        Uranos.biomeRegistry.register(
            Biome(
                "minecraft:swamp",
                Biome.Precipitation.NONE,
                RGBColor(12),
                RGBColor(1),
                RGBColor(1),
                RGBColor(1),
                Biome.MoodSound(0, 0.0, "minecraft:entity.pig.ambient", 0),
                0f,
                0f
            )
        )

        Uranos.biomeRegistry.register(
            Biome(
                "minecraft:swamp_hills",
                Biome.Precipitation.NONE,
                RGBColor(12),
                RGBColor(1),
                RGBColor(1),
                RGBColor(1),
                Biome.MoodSound(0, 0.0, "minecraft:entity.pig.ambient", 0),
                0f,
                0f
            )
        )
    }

    override suspend fun onEnable() {
        enableOptifineFix()

        val dimension = Dimension(
            "test:test",
            true,
            0.0f,
            true
        )

        val biome = Biome(
            "test:test",
            Biome.Precipitation.NONE,
            RGBColor(0xFF0000),
            RGBColor(0x000000),
            RGBColor(0x00FF00),
            RGBColor(0x444444),
            Biome.MoodSound(2, 0.0, "minecraft:entity.pig.ambient", 1),
            0.0f,
            10f
        )

        Uranos.dimensionRegistry.register(dimension)
        Uranos.biomeRegistry.register(biome)

        val world = AnvilWorld(dimension, true, null)
        Uranos.worldRegistry.register(world.internals)

        world.getVoxelsInCuboid(VoxelLocation.of(16, 0, 16), VoxelLocation.of(-16, 0, -16)).forEach {
            it.block = if (it.location.x % 16 == 0 || it.location.z % 16 == 0) GreenWoolBlock() else RedWoolBlock()
        }

        world.getVoxelsInSphere(VoxelLocation.of(20, 50, 20), 40.0).forEach { it.block = RedWoolBlock() }
        world.getChunk(Chunk.Key(0, 0)).setBiome(0, 0, 0, biome)

        Uranos.eventBus.on<ServerListInfoRequestEvent> { event ->
            event.response = ServerListInfo("1.16.4", 754, TextComponent of "Test", 10, 0, emptyList())
        }

        Uranos.eventBus.on<SessionAfterLoginEvent> { event ->
            event.gameMode = GameMode.CREATIVE
            event.canFly = true
            event.flying = true
            event.initialWorldAndLocation = VoxelLocation
                .of(0, 2, 0)
                .atTopCenter()
                .inside(world)

            Uranos.scheduler.executeRepeating(secondsToTicks(1)) {
                for (player in Uranos.players) {
                    player.playerListName = TextComponent(player.name, color = ChatColor.NAMED_COLORS.values.random())
                }
            }
        }

        // Not showing up yet because no metadata is sent
        val entity = Uranos.create<MinecartEntity>()
        entity.position = Position(0.0, 4.0, 0.0)
        entity.setWorld(world)

        var x = 0f
        var y = -90f
        Uranos.scheduler.executeRepeating(1) {
            runInServerThread {
                entity.yaw = x
                entity.pitch = y
            }

            x += 5f
            y += 5f
            if (x == 360f) x = 0f
            if (y == 90f) y = -90f
        }
    }
}
