package space.uranos.testplugin.anvil

import com.google.common.cache.LoadingCache
import space.uranos.player.Player
import space.uranos.util.collections.createWeakValuesLoadingCache
import space.uranos.world.*
import space.uranos.world.block.AirBlock

class AnvilChunk(world: AnvilWorld, key: Key) : Chunk(world, key) {
    // TODO: Implement light

    private val biomes = Array(ChunkData.BIOME_AREAS_IN_CHUNK) { Biome.PLAINS }

    override fun getData(player: Player?): ChunkData {
        return ChunkData(
            sections.map { section -> if (section.blocks.all { it == AirBlock }) null else section.blocks }
                .toTypedArray(),
            biomes
        )
    }

    override fun getLightData(player: Player?): ChunkLightData {
        return ChunkLightData(
            sections.map { it.skyLightValues }.toTypedArray(),
            sections.map { it.blockLightValues }.toTypedArray()
        )
    }

    override fun setBiome(x: Int, y: Int, z: Int, biome: Biome) {
        biomes[y * 16 + z * 4 + x] = biome
    }

    val sections: Array<AnvilChunkSection> = Array(16) { AnvilChunkSection() }

    // Keys are relative to the world
    private val voxels: LoadingCache<VoxelLocation, Voxel> =
        createWeakValuesLoadingCache { location -> AnvilVoxel(location, world) }

    override fun getVoxelRelativeToChunk(location: VoxelLocation): Voxel {
        return voxels.get(key.translateChunkToWorld(location))
    }

    override fun getVoxel(location: VoxelLocation): Voxel {
        return voxels.get(location)
    }
}
