package space.uranos.testplugin.anvil

import com.google.common.cache.CacheBuilder
import com.google.common.cache.CacheLoader
import com.google.common.cache.LoadingCache
import kotlinx.coroutines.CoroutineDispatcher
import space.uranos.entity.Entity
import space.uranos.util.newSingleThreadDispatcher
import space.uranos.world.Chunk
import space.uranos.world.Dimension
import space.uranos.world.World
import java.util.*

class AnvilWorld(
    override val dimension: Dimension,
    override val isFlat: Boolean,
    override val seed: Long?,
) : World {
    override val dispatcher: CoroutineDispatcher = newSingleThreadDispatcher("AnvilWorld")
    override val loadedChunks: Map<Chunk.Key, AnvilChunk> get() = chunks.asMap().filter { (_, chunk) -> chunk.loaded }

    private val chunks: LoadingCache<Chunk.Key, AnvilChunk> = CacheBuilder.newBuilder()
        .build(object : CacheLoader<Chunk.Key, AnvilChunk>() {
            override fun load(key: Chunk.Key): AnvilChunk {
                return AnvilChunk(this@AnvilWorld, key)
            }
        })

    override fun getChunk(key: Chunk.Key): AnvilChunk = chunks.get(key)

    override val entities = mutableSetOf<Entity>()

    val internals = object : World.Internals {
        override val world: World = this@AnvilWorld

        override fun addEntity(entity: Entity) {
            entities.add(entity)
        }

        override fun removeEntity(entity: Entity) {
            entities.remove(entity)
        }
    }
}
