package space.uranos

open class Registry<T: RegistryItem> {
    protected val internalItems = HashMap<String, T>()
    val items: Map<String, T> = internalItems

    /**
     * Adds [item] to [items].
     *
     * @return false when there is already an item with this ID, even if it is [item]. This is because otherwise this
     * function would need to be blocking or suspending.
     */
    open fun register(item: T): Boolean = internalItems.putIfAbsent(item.id, item) == null

    /**
     * Removes [item] from [items].
     *
     * @return true when the item was contained in [items].
     */
    open fun unregister(item: T): Boolean = unregister(item.id)

    /**
     * Removes the item with [id] from [items].
     *
     * @return true when the item was contained in [items].
     */
    open fun unregister(id: String): Boolean = internalItems.remove(id) != null
}

interface RegistryItem {
    val id: String
}
