package space.uranos.chat

import com.squareup.moshi.FromJson
import com.squareup.moshi.ToJson
import space.uranos.util.RGBColor

@Suppress("ClassName")
sealed class ChatColor(val stringRepresentation: String) {
    object BLACK : ChatColor("black")
    object DARK_BLUE : ChatColor("dark_blue")
    object DARK_GREEN : ChatColor("dark_green")
    object DARK_AQUA : ChatColor("dark_aqua")
    object DARK_RED : ChatColor("dark_red")
    object PURPLE : ChatColor("dark_purple")
    object GOLD : ChatColor("gold")
    object GRAY : ChatColor("gray")
    object DARK_GRAY : ChatColor("dark_gray")
    object BLUE : ChatColor("blue")
    object GREEN : ChatColor("green")
    object AQUA : ChatColor("aqua")
    object RED : ChatColor("red")
    object PINK : ChatColor("light_purple")
    object YELLOW : ChatColor("yellow")
    object WHITE : ChatColor("white")

    class Custom(color: RGBColor) :
        ChatColor("#${color.rgb.toString(16).padStart(6, '0')}")

    override fun toString(): String = stringRepresentation

    companion object {
        val NAMED_COLORS by lazy {
            setOf(
                BLACK,
                DARK_BLUE,
                DARK_GREEN,
                DARK_AQUA,
                DARK_RED,
                PURPLE,
                GOLD,
                GRAY,
                DARK_GRAY,
                BLUE,
                GREEN,
                AQUA,
                RED,
                PINK,
                YELLOW,
                WHITE
            ).map { it.stringRepresentation to it }.toMap()
        }

        fun fromString(value: String): ChatColor =
            run {
                if (value.startsWith("#")) RGBColor.fromString(value.removePrefix("#"))?.let { Custom(it) }
                else NAMED_COLORS[value]
            } ?: throw IllegalArgumentException("There is no color named '${value}'")
    }

    object JSONAdapter {
        @ToJson
        fun toJson(value: ChatColor) = value.toString()

        @FromJson
        fun fromJson(value: String) = fromString(value)
    }
}
