package space.uranos.entity

import kotlin.reflect.KClass

interface EntityType<T : Entity> {
    val interfaceType: KClass<T>
    val numericID: Int
    val id: String
    val width: Float
    val height: Float

    companion object {
        /**
         * All entity types, sorted by their numeric ID in ascending order.
         */
        val all: Collection<EntityType<*>> = ENTITY_TYPES

        val byID: Map<String, EntityType<*>> = all.map { it.id to it }.toMap()
        fun byID(id: String) = byID[id]

        private val byInterfaceTypeMap: Map<KClass<out Entity>, EntityType<*>> =
            ENTITY_TYPES.map { it.interfaceType to it }.toMap()

        @Suppress("UNCHECKED_CAST")
        fun <T : Entity> byInterfaceType(interfaceType: KClass<T>): EntityType<T> =
            byInterfaceTypeMap[interfaceType] as EntityType<T>
    }
}
