package space.uranos.entity

import space.uranos.CoordinatePart
import space.uranos.Vector
import space.uranos.world.VoxelLocation
import space.uranos.world.World
import kotlin.math.roundToInt

/**
 * A combination of x, y and z coordinates.
 */
data class Position(val x: Double, val y: Double, val z: Double) {
    /**
     * Converts this Position to a VoxelLocation by converting [x], [y] and [z] to integers using [Double.toInt],
     * in contrast to [roundToBlock] which uses [Double.roundToInt].
     */
    fun toVoxelLocation(): VoxelLocation = VoxelLocation(x.toInt(), y.toInt().toUByte(), z.toInt())

    /**
     * Converts this Position to a VoxelLocation by converting [x], [y] and [z] to integers using [Double.roundToInt],
     * in contrast to [toVoxelLocation] which uses [Double.toInt].
     */
    fun roundToBlock(): VoxelLocation = VoxelLocation(x.roundToInt(), y.roundToInt().toUByte(), z.roundToInt())

    fun asVector() = Vector(x, y, z)
    infix fun inside(world: World): Pair<World, Position> = world to this

    operator fun get(part: CoordinatePart): Double = when (part) {
        CoordinatePart.X -> x
        CoordinatePart.Y -> y
        CoordinatePart.Z -> z
    }

    companion object {
        val ZERO = Position(0.0, 0.0, 0.0)
    }
}
