package space.uranos.net

import com.squareup.moshi.Json
import com.squareup.moshi.JsonClass
import space.uranos.chat.TextComponent
import java.util.UUID

/**
 * Used in the [ServerListInfoRequestEvent][space.uranos.net.event.ServerListInfoRequestEvent].
 *
 * @param versionName The name of the Minecraft version the server uses.
 * @param protocolVersion The number of the [protocol version](https://wiki.vg/Protocol_version_numbers) used by the server.
 * @param description The description of the server. Although this is a [TextComponent],
 * [LegacyFormattingCode][space.uranos.chat.LegacyFormattingCode]s must be used.
 * @param maxPlayers The maximum amount of players that can join the server. This is only visual.
 * @param onlinePlayers The amount of online players.
 * @param playerListSample The players shown when hovering over the player count.
 * @param favicon The favicon of the server. This must be a base64 encoded 64x64 PNG image.
 */
data class ServerListInfo(
    val versionName: String,
    val protocolVersion: Int,
    val description: TextComponent,
    val maxPlayers: Int,
    val onlinePlayers: Int,
    val playerListSample: List<PlayerListSampleEntry>,
    val favicon: String? = null
) {
    /**
     * @param name The name of the player. You can use [LegacyFormattingCode][space.uranos.chat.LegacyFormattingCode]s.
     * @param uuid The UUID of the player as a string. You can use a random UUID as it does not get validated by the
     * Minecraft client
     */
    @JsonClass(generateAdapter = true)
    data class PlayerListSampleEntry(val name: String, @property:Json(name = "id") val uuid: UUID)
}
