package space.uranos.net.event

import space.uranos.entity.Position
import space.uranos.event.Cancellable
import space.uranos.net.Session
import space.uranos.player.GameMode
import space.uranos.player.Player
import space.uranos.world.World

/**
 * Emitted after a [Session] finished logging in.
 *
 * If the event is cancelled, the session is disconnected.
 */
class SessionAfterLoginEvent(override val target: Session) : SessionEvent(), Cancellable {
    override var cancelled = false

    /**
     * Whether the respawn screen is shown when the player dies.
     */
    var respawnScreenEnabled: Boolean = true

    /**
     * The location where the player will spawn. If this is null after all handlers ran, the session is disconnected.
     */
    var initialWorldAndLocation: Pair<World, Position>? = null

    var headYaw: Float = 0f
    var headPitch: Float = 0f

    var maxViewDistance: Int = 32
        set(value) {
            if (value in 2..32) field = value
            else throw IllegalArgumentException("maxViewDistance must be in 2..32")
        }

    /**
     * Whether the player should see his hearts as they look in hardcore mode.
     */
    var hardcoreHearts = false

    /**
     * See [Player.gameMode].
     */
    var gameMode: GameMode = GameMode.SURVIVAL

    /**
     * See [Player.selectedHotbarSlot].
     */
    var selectedHotbarSlot: Int = 0
        set(value) {
            if (value in 0..8) field = value
            else throw IllegalArgumentException("selectedHotbarSlot must be in 0..8")
        }

    /**
     * See [Player.reducedDebugInfo].
     */
    var reducedDebugInfo: Boolean = false

    /**
     * See [Player.vulnerable].
     */
    var invulnerable: Boolean = false

    /**
     * See [Player.canFly].
     */
    var canFly: Boolean = false

    /**
     * See [Player.flying].
     */
    var flying: Boolean = false

    /**
     * See [Player.flyingSpeed].
     *
     * Defaults to [Player.DEFAULT_FLYING_SPEED].
     */
    var flyingSpeed: Float = Player.DEFAULT_FLYING_SPEED

    /**
     * See [Player.fieldOfView].
     *
     * Defaults to [Player.DEFAULT_FIELD_OF_VIEW].
     */
    var fieldOfView: Float = Player.DEFAULT_FIELD_OF_VIEW
}
