package space.uranos.player

import space.uranos.chat.TextComponent
import space.uranos.entity.PlayerEntity
import space.uranos.net.Session
import space.uranos.world.Chunk
import space.uranos.world.VoxelLocation
import java.util.UUID

interface Player {
    val scope get() = session.scope

    /**
     * The session of this player.
     */
    val session: Session

    val entity: PlayerEntity

    /**
     * The name of this player.
     */
    val name: String

    /**
     * The [UUID] of this player.
     */
    val uuid: UUID

    /**
     * The [GameMode] of this player.
     */
    var gameMode: GameMode

    /**
     * The settings this client sent in the `ClientSettings` packet.
     */
    val settings: Settings

    data class Settings(
        val viewDistance: Byte,
        val locale: String,
        val mainHand: Hand,
        val enabledSkinParts: SkinPartsConfiguration,
        val chatColorsEnabled: Boolean,
        val chatMode: ChatMode
    )

    /**
     * The index of the hotbar slot which is currently selected.
     * Must be in `0..8`.
     */
    var selectedHotbarSlot: Int

    /**
     * Corresponds to the `reducedDebugInfo` gamerule in vanilla.
     *
     * @see <a href="https://minecraft.gamepedia.com/Debug_screen#Legend">https://minecraft.gamepedia.com/Debug_screen#Legend</a>
     */
    var reducedDebugInfo: Boolean

    /**
     * Whether the player can take damage through other entities or the world.
     */
    var vulnerable: Boolean

    /**
     * Whether the player can start flying by itself.
     */
    var canFly: Boolean

    /**
     * Whether the player is currently flying.
     */
    var flying: Boolean

    /**
     * The default is [DEFAULT_FLYING_SPEED].
     */
    var flyingSpeed: Float

    /**
     * Modifies the field of view, like a speed potion.
     *
     * The default is [DEFAULT_FIELD_OF_VIEW].
     */
    var fieldOfView: Float

    var playerListName: TextComponent?
    var compassTarget: VoxelLocation

    val currentlyViewedChunks: Collection<Chunk>

    companion object {
        const val DEFAULT_FIELD_OF_VIEW = 0.1f
        const val DEFAULT_FLYING_SPEED = 0.05f
    }
}
