package space.uranos.util

import kotlin.reflect.KProperty

class MemoizedDelegate<T>(private val key: () -> Any?, private val initializer: () -> T) {
    private object UNINITIALIZED

    private var lastDependingValue: Any? = null
    private var value: Any? = UNINITIALIZED

    operator fun getValue(thisRef: Any?, property: KProperty<*>): T {
        val currentDependingValue = key()

        if (value == UNINITIALIZED || (lastDependingValue != currentDependingValue)) {
            value = initializer()
            lastDependingValue = currentDependingValue
        }

        @Suppress("UNCHECKED_CAST")
        return value as T
    }
}

fun <T> memoized(dependingGetter: () -> Any?, initializer: () -> T) = MemoizedDelegate(dependingGetter, initializer)
