package space.uranos.util

import com.squareup.moshi.FromJson
import com.squareup.moshi.JsonQualifier
import com.squareup.moshi.Moshi
import com.squareup.moshi.ToJson
import space.uranos.chat.ChatColor
import space.uranos.chat.ChatComponent
import space.uranos.chat.TextComponent
import java.util.UUID

fun Moshi.toJson(value: Map<*, *>): String = adapter(Map::class.java).toJson(value)

object UUIDAdapter {
    @ToJson
    fun toJson(value: UUID) = value.toString()

    @FromJson
    fun fromJson(json: String): UUID = UUID.fromString(json)
}

@Retention
@JsonQualifier
/**
 * **Only applicable to [UUID]s.**
 */
annotation class WithoutHyphens

object WithoutHyphensQualifierAdapter {
    @FromJson
    @WithoutHyphens
    fun fromJson(value: String) = value.toUUID()

    @ToJson
    fun toJson(@WithoutHyphens value: UUID) = value.toStringWithoutHyphens()
}

/**
 * An instance of [Moshi] which should be used for all JSON (de)serialization.
 *
 * @see <a href="https://github.com/square/moshi/blob/master/README.md">The Moshi documentation</a>
 */
val moshi = Moshi.Builder()
    .add(UUIDAdapter)
    .add(ChatComponent.JSONAdapter)
    .add(ChatColor.JSONAdapter)
    .add(TextComponent.JSONAdapter)
    .add(WithoutHyphensQualifierAdapter)
    .build()!!
