package space.uranos.util

class RGBColor(rgb: Int) {
    val rgb = rgb and 0xFFFFFF

    constructor(
        red: UByte,
        green: UByte,
        blue: UByte
    ) : this((red.toInt() shl 16) and (green.toInt() shl 8) and blue.toInt())

    constructor(javaColor: java.awt.Color) : this(javaColor.rgb)

    val red: UByte by lazy { ((rgb and 0xFF0000) shr 16).toUByte() }
    val green: UByte by lazy { ((rgb and 0x00FF00) shr 8).toUByte() }
    val blue: UByte by lazy { (rgb and 0x0000FF).toUByte() }

    fun copy(red: UByte = this.red, green: UByte = this.green, blue: UByte = this.blue) = RGBColor(red, green, blue)

    override fun toString() = "0x${rgb.toString(16).toUpperCase().padStart(6, '0')}"

    override fun equals(other: Any?): Boolean = (other == this) || (other is RGBColor && other.rgb == this.rgb)
    override fun hashCode(): Int = rgb

    companion object {
        fun fromString(string: String): RGBColor? =
            runCatching { string.removePrefix("0x").toInt(16) }.getOrNull()?.let { RGBColor(it) }
    }
}
