package space.uranos.util

import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.launch
import space.uranos.Uranos
import space.uranos.server.Server

/**
 * Suspends for [ticks].
 */
suspend inline fun delayTicks(ticks: Long) {
    Uranos.scheduler.runAfter(ticks) {}
}

/**
 * Launches a new coroutine and delays its execution by 1 tick.
 *
 * @see [CoroutineScope.launch]
 */
inline fun CoroutineScope.launchNextTick(crossinline block: suspend () -> Unit) = launch {
    delayTicks(1)
    block()
}

@Suppress("NOTHING_TO_INLINE")
inline fun nextTick(noinline block: suspend () -> Unit) {
    Uranos.scheduler.executeAfter(1, block)
}

/**
 * Returns the amount of ticks (without decimal places) that are equal to [seconds].
 */
fun secondsToTicks(seconds: Double): Long = (seconds * Server.TICKS_PER_SECOND).toLong()

/**
 * Returns the amount of ticks that are equal to [seconds].
 */
fun secondsToTicks(seconds: Long): Long = seconds * Server.TICKS_PER_SECOND

/**
 * Returns the amount of ticks (without decimal places) that are equal to [milliseconds].
 */
fun msToTicks(milliseconds: Double): Long = ((milliseconds / 1000) * Server.TICKS_PER_SECOND).toLong()

/**
 * Returns the amount of ticks that are equal to [milliseconds].
 */
fun msToTicks(milliseconds: Long): Long = (milliseconds / 1000) * Server.TICKS_PER_SECOND
