package space.uranos.util.collections

import com.google.common.cache.CacheBuilder
import com.google.common.cache.CacheLoader
import com.google.common.cache.LoadingCache

/**
 * Creates a LoadingCache with weak values.
 *
 * @param loader The function invoked every time a value is not yet in the cache.
 */
fun <K, V> createWeakValuesLoadingCache(loader: (key: K) -> V): LoadingCache<K, V> {
    return CacheBuilder
        .newBuilder()
        .weakValues()
        .build(object : CacheLoader<K, V>() {
            override fun load(key: K): V = loader(key)
        })
}
