package space.uranos.world

import space.uranos.RegistryItem
import space.uranos.util.RGBColor

data class Biome(
    override val id: String,
    val precipitation: Precipitation,
    val skyColor: RGBColor,
    val waterFogColor: RGBColor,
    val fogColor: RGBColor,
    val waterColor: RGBColor,
    val moodSound: MoodSound,
    /**
     * Has an effect on grass and foliage color
     */
    val temperature: Float,
    /**
     * Has an effect on grass and foliage color
     */
    val downfall: Float
): RegistryItem {
    /**
     * The numeric ID of this biome. Is set when this biome is registered in the biome registry.
     */
    var numericID: Int? = null; internal set

    enum class Precipitation {
        NONE,
        RAIN,
        SNOW
    }

    data class MoodSound(
        /**
         * The amount of ticks after which the sound plays.
         */
        val delay: Int, // TODO: Confirm this
        val offset: Double, // TODO: Find out what this does
        val sound: String,
        /**
         * Determines the cubic range of possible positions to play the mood sound.
         * The player is at the center of the cubic range, and the edge length is `2 * maxDistance + 1`.
         */
        val maxDistance: Int
    )

    companion object {
        /**
         * The vanilla "plains" biome.
         *
         * This is automatically registered in the biome registry because otherwise the client crashes.
         * It cannot be unregistered.
         */
        val PLAINS = Biome(
            "minecraft:plains",
            Precipitation.RAIN,
            RGBColor(7907327),
            RGBColor(329011),
            RGBColor(12638463),
            RGBColor(4159204),
            MoodSound(
                6000,
                2.0,
                "minecraft:ambient.cave",
                8
            ),
            0.8f,
            0.4f
        )
    }
}
