package space.uranos.world

data class ChunkLightData(
    /**
     * The outer array must have 18 items representing the chunk sections.
     *
     * The inner arrays contain the block light values (x, z, y) of a chunk section and must have a size of 2048.
     * Every byte stores two values (4 bits each). Values for blocks with an even x coordinate are packed into the
     * least-significant bits.
     *
     * If the light values in a section should be left as is, null can be used.
     */
    val blockLightValues: Array<UByteArray?>,

    /**
     * The outer array must have 18 items representing the chunk sections.
     *
     * The inner arrays contain the sky light values (x, z, y) of a chunk section and must have a size of 2048.
     * Every byte stores two values (4 bits each). Values for blocks with an even x coordinate are packed into the
     * least-significant bits.
     *
     * If the light values in a section should be left as is, null can be used.
     */
    val skyLightValues: Array<UByteArray?>
) {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as ChunkLightData

        if (!blockLightValues.contentDeepEquals(other.blockLightValues)) return false
        if (!skyLightValues.contentDeepEquals(other.skyLightValues)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = blockLightValues.contentDeepHashCode()
        result = 31 * result + skyLightValues.contentDeepHashCode()
        return result
    }

    override fun toString(): String = "ChunkLightData([omitted])"
}
