package space.uranos.world.block

import space.uranos.world.block.Block.Companion.material
import kotlin.reflect.KClass

/**
 * A block of [material].
 *
 * Blocks should always be **immutable**.
 */
abstract class Block internal constructor() {
    companion object {
        /**
         * See [DaylightDetectorBlock] for an example on how this function should be used.
         */
        internal inline fun <reified T : Block> material(
            numericID: Int,
            id: String,
            firstStateID: Int,
            hardness: Float,
            transparent: Boolean,
            filteredLight: Int,
            emittedLight: Int,
            collisionShape: CollisionShape
        ) = object : Material<T> {
            override val blockClass: KClass<T> = T::class
            override val codec: BlockCodec<T> = BlockCodec(blockClass, numericID, firstStateID)
            override val id: String = id
            override val hardness: Float = hardness
            override val transparent: Boolean = transparent
            override val filteredLight: Int = filteredLight
            override val emittedLight: Int = emittedLight
            override val collisionShape: CollisionShape = collisionShape
        }
    }
}

@Suppress("UNCHECKED_CAST")
fun <T : Block> T.material(): Material<T> = Material.byClass(this::class) as Material<T>
