package space.uranos.nbt

import java.io.DataInputStream
import java.io.DataOutputStream

class NBT internal constructor(private val map: MutableMap<String, Any>) {
    /**
     * Creates an instance with no data.
     */
    constructor() : this(mutableMapOf())

    /**
     * The current data.
     */
    val data: Map<String, Any> get() = map

    /**
     * Gets the value for [name] as [T].
     */
    inline fun <reified T> get(name: String) = data[name] as T

    fun set(name: String, value: Byte) {
        map[name] = value
    }

    fun set(name: String, value: Short) {
        map[name] = value
    }

    fun set(name: String, value: Int) {
        map[name] = value
    }

    fun set(name: String, value: Long) {
        map[name] = value
    }

    fun set(name: String, value: Float) {
        map[name] = value
    }

    fun set(name: String, value: Double) {
        map[name] = value
    }

    fun set(name: String, value: ByteArray) {
        map[name] = value
    }

    fun set(name: String, value: String) {
        map[name] = value
    }

    fun set(name: String, value: Collection<*>) {
        map[name] = value
    }

    fun set(name: String, value: NBT) {
        map[name] = value
    }

    fun set(name: String, value: LongArray) {
        map[name] = value
    }

    fun setAsByte(name: String, value: Boolean) {
        map[name] = if(value) 1.toByte() else 0.toByte()
    }

    inline fun set(name: String, nested: NBTBuilderContext.() -> Unit) = set(name, buildNBT(nested))

    fun write(destination: DataOutputStream, name: String? = null) {
        if (name == null) NBTCompound.writeValue(destination, this)
        else NBTCompound.writeNamedTag(destination, name, this)
    }

    fun toSNBT(pretty: Boolean) = NBTCompound.toSNBT(this, pretty)
    override fun toString() = toSNBT(true)

    companion object {
        fun read(source: DataInputStream): NBT {
            return NBTCompound.readTopLevelValue(source)
        }
    }
}

inline fun buildNBT(init: NBTBuilderContext.() -> Unit) = NBT().also { NBTBuilderContext(it).init() }
