package space.uranos.nbt

import java.io.DataInputStream
import java.io.DataOutputStream
import java.io.IOException

object NBTList : NBTType<Collection<*>>(Collection::class, 9) {
    override fun toSNBT(value: Collection<*>, pretty: Boolean): String {
        val multiline = pretty && value.size > 1
        val separator = "," + (if (multiline) "\n" else if (pretty) " " else "")

        val items = value
            .joinToString(separator) { of(it!!)!!.toSNBT(it, pretty) }
            .run {
                if (multiline) prependIndent("  ")
                else this
            }

        return "[" + stringIf(multiline, "\n") + items + stringIf(multiline, "\n") + "]"
    }

    override fun readValue(source: DataInputStream): List<Any> {
        val typeID = source.readByte()
        val length = source.readInt()

        if (length == 0) return emptyList()

        val type = byID[typeID]
            ?: throw IOException("The NBT data contains an unknown type ID: $typeID")

        return (1..length).map { type.readValue(source) }
    }

    override fun writeValue(destination: DataOutputStream, value: Collection<*>) {
        @Suppress("UNCHECKED_CAST")
        value as Collection<Any>

        if (value.isEmpty()) {
            destination.writeByte(END_TAG_ID.toInt())
            destination.writeByte(0)
        } else {
            val type = of(value.first())
                ?: throw IllegalArgumentException("The type of value cannot be represented as NBT")

            destination.writeByte(type.id.toInt())
            destination.writeInt(value.size)

            value.forEach { type.writeValue(destination, it) }
        }
    }
}
