package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import space.uranos.Uranos
import space.uranos.nbt.buildNBT
import space.uranos.net.MinecraftProtocolDataTypes.writeNBT
import space.uranos.net.MinecraftProtocolDataTypes.writeString
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.packet.OutgoingPacketCodec
import space.uranos.util.sha256
import space.uranos.util.toByteArray
import space.uranos.util.toLong

object JoinGamePacketCodec : OutgoingPacketCodec<JoinGamePacket>(0x24, JoinGamePacket::class) {
    override fun JoinGamePacket.encode(dst: ByteBuf) {
        dst.writeInt(entityID)
        dst.writeBoolean(hardcore)
        dst.writeByte(gameMode.numericID)
        dst.writeByte(-1) // "Previous game mode"
        dst.writeVarInt(1)
        dst.writeString("uranos:world")

        val dimensionsByID = Uranos.dimensionRegistry.items.values.map { dimension ->
            dimension.id to buildNBT {
                "natural" setAsByte !dimension.compassesSpinRandomly
                "ambient_light" set dimension.ambientLight
                "has_skylight" setAsByte dimension.hasSkylight

                // Not known what this does
                "effects" set "minecraft:overworld"

                // These values do not actually change something client-sided
                "ultrawarm" setAsByte false
                "has_ceiling" setAsByte false
                "has_raids" setAsByte true
                "logical_height" set 255
                "coordinate_scale" set 1.toFloat()
                "bed_works" setAsByte true
                "fixed_light" setAsByte false
                "infiniburn" set ""
                "respawn_anchor_works" setAsByte false
                "piglin_safe" setAsByte false
            }
        }.toMap()

        // TODO: Cache this
        val dimensions = buildNBT {
            "minecraft:dimension_type" {
                "type" set "minecraft:dimension_type"
                "value" set dimensionsByID.entries.mapIndexed { index, (id, dimension) ->
                    buildNBT {
                        "name" set id
                        "id" set index
                        "element" set dimension
                    }
                }
            }

            "minecraft:worldgen/biome" {
                "type" set "minecraft:worldgen/biome"
                "value" set Uranos.biomeRegistry.items.values.map { biome ->
                    buildNBT {
                        "name" set biome.id
                        "id" set biome.numericID!!
                        "element" {
                            "precipitation" set biome.precipitation.name.toLowerCase()
                            "temperature" set biome.temperature
                            "downfall" set biome.downfall
                            "effects" {
                                "sky_color" set biome.skyColor.rgb
                                "water_fog_color" set biome.waterFogColor.rgb
                                "fog_color" set biome.fogColor.rgb
                                "water_color" set biome.waterColor.rgb
                                "mood_sound" {
                                    "tick_delay" set biome.moodSound.delay
                                    "offset" set biome.moodSound.offset
                                    "sound" set biome.moodSound.sound
                                    "block_search_extend" set biome.moodSound.maxDistance
                                }
                            }

                            // These values do not actually change something client-sided
                            "depth" set 0f
                            "scale" set 0f
                            "category" set "plains"
                        }
                    }
                }
            }
        }

        dst.writeNBT(dimensions)
        dst.writeNBT(dimensionsByID.getValue(world.dimension.id))

        dst.writeString("uranos:world")
        dst.writeLong(sha256((world.seed ?: 0).toByteArray()).sliceArray(0..7).toLong())

        dst.writeVarInt(0) // max players; not used anymore

        dst.writeVarInt(maxViewDistance)
        dst.writeBoolean(reducedDebugInfo)
        dst.writeBoolean(respawnScreenEnabled)

        dst.writeBoolean(false) // Is debug world
        dst.writeBoolean(world.isFlat)
    }
}
