package space.uranos.net.packet.play

import io.netty.buffer.ByteBuf
import space.uranos.net.MinecraftProtocolDataTypes.writeString
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.packet.OutgoingPacketCodec
import space.uranos.tag.Tag
import kotlin.time.Duration
import kotlin.time.toJavaDuration

object TagsPacketCodec :
    OutgoingPacketCodec<TagsPacket>(0x5B, TagsPacket::class, CacheOptions(3, Duration.INFINITE.toJavaDuration())) {
    private val ORDER = listOf(
        Tag.Type.BLOCKS,
        Tag.Type.ITEMS,
        Tag.Type.FLUIDS,
        Tag.Type.ENTITY_TYPES
    )

    override fun TagsPacket.encode(dst: ByteBuf) {
        val tagsByType = tags.groupBy { it.type }

        ORDER.forEach { type ->
            val tags = tagsByType[type] ?: emptyList()
            dst.writeVarInt(tags.size)

            tags.forEach { tag ->
                dst.writeString(tag.name)
                dst.writeVarInt(tag.values.size)
                tag.numericIDs.forEach { dst.writeVarInt(it) }
            }
        }
    }
}
