package space.uranos.util

import space.uranos.world.Chunk
import space.uranos.world.block.Block
import space.uranos.world.block.Material

fun generateHeightmap(sections: Array<Array<Block>?>, ignoredBlocks: Iterable<Material<*>>): ByteArray {
    val array = ByteArray(Chunk.AREA)

    for (sectionIndex in sections.indices.reversed()) {
        val section = sections[sectionIndex] ?: continue

        for (y in 0 until Chunk.SECTION_HEIGHT) {
            for (z in 0 until Chunk.LENGTH) {
                for (x in 0 until Chunk.LENGTH) {
                    val index = y * Chunk.AREA + z * Chunk.LENGTH + x
                    val block = section[index]

                    if (ignoredBlocks.any { it.blockClass == block::class })
                        array[z * Chunk.LENGTH + x] = (y - 1).toByte()
                }
            }
        }

        // All columns already have a value, so the following sections do not need to be checked anymore
        if (array.none { it == 0.toByte() }) break
    }

    return array
}
