package space.uranos.net.packet

import org.junit.jupiter.api.Test
import space.uranos.net.packet.handshaking.HandshakingProtocol
import space.uranos.net.packet.login.LoginProtocol
import space.uranos.net.packet.play.PlayProtocol
import space.uranos.net.packet.status.StatusProtocol

class ProtocolValidationTest {
    private fun ensureIDUniqueness(codecs: List<PacketCodec<*>>, type: String, protocol: Protocol) {
        codecs.groupBy { it.id }.forEach { (id, c) ->
            if (c.count() > 1) {
                val packetsString = c.joinToString(", ", limit = 4) { it.dataType.simpleName.toString() }
                throw AssertionError(
                    "Multiple $type packets of the ${protocol.name} protocol use the same ID " +
                            "(0x${id.toString(16)}): $packetsString"
                )
            }
        }
    }

    @Test
    fun `all packets have unique IDs in their protocol`() {
        for (protocol in PROTOCOLS) {
            ensureIDUniqueness(protocol.codecs.filterIsInstance<IncomingPacketCodec<*>>(), "incoming", protocol)
            ensureIDUniqueness(protocol.codecs.filterIsInstance<OutgoingPacketCodec<*>>(), "outgoing", protocol)
        }
    }

    companion object {
        val PROTOCOLS = setOf(HandshakingProtocol, LoginProtocol, StatusProtocol, PlayProtocol)
    }
}
