package space.uranos.net.packet.handshaking

import space.uranos.net.packet.IncomingPacket

/**
 * This is the first packet a client should send. Per default, the server immediately switches
 * the [session's currentProtocol][space.uranos.net.Session.currentProtocol] to `STATUS` or `LOGIN`,
 * depending on the value of [loginAttempt].
 *
 * @param protocolVersion The number of the [protocol version](https://wiki.vg/Protocol_version_numbers) used by the client.
 * @param serverAddress Hostname or IP address that was used to connect.
 * @param serverPort Port that was used to connect.
 * @param loginAttempt Whether the server should use the `STATUS` or the `LOGIN` protocol from now on.
 */
data class HandshakePacket(
    val protocolVersion: Int,
    val serverAddress: String,
    val serverPort: Int,
    val loginAttempt: Boolean
) : IncomingPacket()
