package space.uranos.net.packet.login

import space.uranos.net.packet.OutgoingPacket

/**
 * Sent by the server to request authentication and encryption from the client. The client must respond with
 * a [EncryptionResponsePacket].
 *
 * @see <a href="https://wiki.vg/Protocol_Encryption">https://wiki.vg/Protocol_Encryption</a>
 */
data class EncryptionRequestPacket(
    val publicKey: ByteArray,
    val verifyToken: ByteArray
) : OutgoingPacket() {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as EncryptionRequestPacket

        if (!publicKey.contentEquals(other.publicKey)) return false
        if (!verifyToken.contentEquals(other.verifyToken)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = 0
        result = 31 * result + publicKey.contentHashCode()
        result = 31 * result + verifyToken.contentHashCode()
        return result
    }
}
