package space.uranos.net.packet.login

import space.uranos.net.packet.OutgoingPacket

/**
 * Optionally sent by the server to transfer additional, non-standard information required for the login process.
 * The original client will always respond with a [LoginPluginResponsePacket] whose data is null.
 *
 * @param messageID Should be unique to the connection.
 * @param channel Name of the [plugin channel](https://wiki.vg/Plugin_channels).
 * @param data Any data, depending on the channel.
 */
data class LoginPluginRequestPacket(
    val messageID: Int,
    val channel: String,
    val data: ByteArray
) : OutgoingPacket() {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as LoginPluginRequestPacket

        if (messageID != other.messageID) return false
        if (channel != other.channel) return false
        if (!data.contentEquals(other.data)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = messageID
        result = 31 * result + channel.hashCode()
        result = 31 * result + data.contentHashCode()
        return result
    }
}
