package space.uranos.net.packet.play

import space.uranos.net.packet.IncomingPacket

/**
 * Can be used by client-side mods to communicate with a potentially modded server.
 * Minecraft itself [also uses this for some things](https://wiki.vg/Plugin_channels#Channels_internal_to_Minecraft).
 *
 * @param channel Channel in which the message was sent.
 * @param data Data which was sent.
 */
data class IncomingPluginMessagePacket(val channel: String, val data: ByteArray) : IncomingPacket() {
    override fun equals(other: Any?): Boolean {
        if (this === other) return true
        if (javaClass != other?.javaClass) return false

        other as IncomingPluginMessagePacket

        if (channel != other.channel) return false
        if (!data.contentEquals(other.data)) return false

        return true
    }

    override fun hashCode(): Int {
        var result = channel.hashCode()
        result = 31 * result + data.contentHashCode()
        return result
    }
}
