package space.uranos.net.packet.play

import space.uranos.net.packet.OutgoingPacket
import space.uranos.player.GameMode
import space.uranos.world.World

/**
 * Sent by the server after the client logged in.
 *
 * @param entityID ID of the player entity.
 * @param gameMode Game mode of the player.
 * @param world The world in which the player spawns.
 * @param maxViewDistance Maximum view distance allowed by the server.
 * @param reducedDebugInfo Whether the debug screen shows only reduced info.
 * @param respawnScreenEnabled Whether the respawn screen is shown when the player dies.
 */
data class JoinGamePacket(
    val entityID: Int,
    val gameMode: GameMode,
    val hardcore: Boolean,
    val world: World,
    val maxViewDistance: Int,
    val reducedDebugInfo: Boolean,
    val respawnScreenEnabled: Boolean
) : OutgoingPacket() {
    init {
        if (maxViewDistance !in 2..32) throw IllegalArgumentException("viewDistance must be in 2..32")
    }
}
