package space.uranos.util

import space.uranos.entity.*
import space.uranos.net.packet.OutgoingPacket
import space.uranos.net.packet.play.SpawnLivingEntityPacket
import space.uranos.net.packet.play.SpawnObjectEntityPacket
import space.uranos.net.packet.play.SpawnPaintingPacket
import space.uranos.util.numbers.mapToUByte

fun Entity.createSpawnPacket(): OutgoingPacket = when (this) {
    is LivingEntity -> if (this is HasMovableHead) SpawnLivingEntityPacket(
        numericID,
        uuid,
        type,
        position,
        headYaw,
        headPitch,
        headYaw,
        velocity
    ) else SpawnLivingEntityPacket(
        numericID,
        uuid,
        type,
        position,
        if (this is YawRotatable) yaw else 0f,
        if (this is PitchRotatable) pitch else 0f,
        0f,
        velocity
    )
    is ObjectEntity -> SpawnObjectEntityPacket(
        numericID,
        uuid,
        type.numericID,
        position.x,
        position.y,
        position.z,
        if (this is YawRotatable) yaw.mapToUByte(360f) else 0u,
        if (this is PitchRotatable) pitch.mapToUByte(360f) else 0u,
        getDataValue(),
        velocity
    )
    is PaintingEntity -> SpawnPaintingPacket(
        numericID,
        uuid,
        motive,
        calculateCenterLocation(),
        facing
    )
    else -> throw IllegalArgumentException("Unknown entity type")
}

fun ObjectEntity.getDataValue(): Int = when (this) {
    is ItemEntity -> 1
    is MinecartEntity -> 2
    // TODO: Add remaining
    else -> throw IllegalArgumentException("Unknown entity type")
}
