package space.uranos.config

import space.uranos.logging.Logger
import space.uranos.net.packet.Packet
import java.time.Duration

data class UranosConfig(
    val port: Int,
    val host: String,
    val ignoreClientCausedErrors: Boolean,
    val authenticateAndEncrypt: Boolean,
    val developmentMode: Boolean,
    val packetCompressionThreshold: Int,
    val timeout: Duration,
    val packetsBufferSize: Int,
    val pingUpdateInterval: Duration,
    val logging: LoggingConfig
) {
    data class LoggingConfig(
        val minLevel: Logger.Level,
        val events: Boolean,
        val packets: List<String>,
        val isBlacklist: Boolean
    ) {
        fun shouldLog(packet: Packet): Boolean {
            val contained = packets.contains(packet::class.simpleName?.removeSuffix("Packet"))
            return if (isBlacklist) !contained else contained
        }
    }
}
