package space.uranos.logging

import de.moritzruth.khalk.Khalk
import java.io.PrintStream
import java.text.SimpleDateFormat
import java.util.Date

object UranosLoggingOutputProvider : LoggingOutputProvider {
    private val dateFormat = SimpleDateFormat("HH:mm:ss.SSS")

    override fun log(
        printThreadName: Boolean,
        loggerName: String,
        level: Logger.Level,
        message: String,
        throwable: Throwable?
    ) {
        if (!level.isEnabled) return
        val time = Date()

        val stream: PrintStream = if (level.isGreaterOrEqualThan(Logger.Level.ERROR)) System.err else System.out
        val khalk = Khalk()
        val levelColor = when (level) {
            Logger.Level.ERROR -> khalk.redBright
            Logger.Level.WARN -> khalk.yellowBright
            Logger.Level.INFO -> khalk.greenBright
            Logger.Level.DEBUG -> khalk.cyanBright
            Logger.Level.TRACE -> khalk.gray
            else -> khalk
        }

        val parts = mutableListOf<String>()

        parts.add(khalk.gray { dateFormat.format(time) })
        parts.add(levelColor { level.toString().padEnd(5) })
        if (printThreadName) parts.add(khalk.bold { Thread.currentThread().name })
        parts.add(levelColor.inverse { " $loggerName " })
        parts.add(levelColor { "❯" })
        parts.add(message)

        stream.println(parts.joinToString(" "))
        throwable?.printStackTrace(stream)
    }
}
