package space.uranos.net

import io.netty.buffer.ByteBuf
import io.netty.buffer.Unpooled
import io.netty.channel.ChannelHandlerContext
import io.netty.handler.codec.ByteToMessageCodec
import space.uranos.net.MinecraftProtocolDataTypes.readVarInt
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import java.util.zip.Deflater
import java.util.zip.Inflater

class CompressionCodec(private val threshold: Int) : ByteToMessageCodec<ByteBuf>() {
    private val deflater = Deflater(3)
    private val deflationBuffer = ByteArray(8192)
    private val inflater = Inflater()

    override fun encode(ctx: ChannelHandlerContext, msg: ByteBuf, out: ByteBuf) {
        if (msg.readableBytes() >= threshold) {
            out.writeVarInt(msg.readableBytes())

            deflater.setInput(msg.nioBuffer())
            deflater.finish()

            while (!deflater.finished()) {
                out.writeBytes(deflationBuffer, 0, deflater.deflate(deflationBuffer))
            }

            deflater.reset()
        } else {
            out.writeVarInt(0)
            out.writeBytes(msg)
        }
    }

    override fun decode(ctx: ChannelHandlerContext, msg: ByteBuf, out: MutableList<Any>) {
        val length = msg.readVarInt()
        val realLength = msg.readableBytes()

        if (length == 0) {
            out.add(msg.readRetainedSlice(realLength))
        } else {
            val buffer = Unpooled.buffer(length)

            inflater.setInput(msg.nioBuffer())
            inflater.inflate(buffer.nioBuffer())
            inflater.reset()

            out.add(buffer)
        }
    }
}
