package space.uranos.net

import space.uranos.net.packet.IncomingPacket
import space.uranos.net.packet.handshaking.HandshakingProtocol
import space.uranos.net.packet.handshaking.HandshakingProtocolHandler
import space.uranos.net.packet.login.LoginProtocol
import space.uranos.net.packet.login.LoginProtocolHandler
import space.uranos.net.packet.play.PlayProtocol
import space.uranos.net.packet.play.PlayProtocolHandler
import space.uranos.net.packet.status.StatusProtocol
import space.uranos.net.packet.status.StatusProtocolHandler
import kotlin.reflect.KClass

open class ProtocolPacketReceivedEventHandler(private val handlers: Map<out KClass<out IncomingPacket>, PacketReceivedEventHandler<out IncomingPacket>>) {
    suspend fun handle(session: UranosSession, packet: IncomingPacket) {
        @Suppress("UNCHECKED_CAST")
        val handler: PacketReceivedEventHandler<IncomingPacket>? =
            handlers[packet::class] as PacketReceivedEventHandler<IncomingPacket>?
        handler?.handle(session, packet)
    }
}

abstract class PacketReceivedEventHandler<T : IncomingPacket> {
    abstract suspend fun handle(session: UranosSession, packet: T)

    companion object {
        fun <T : IncomingPacket> of(fn: suspend (session: UranosSession, packet: T) -> Unit) =
            object : PacketReceivedEventHandler<T>() {
                override suspend fun handle(session: UranosSession, packet: T) = fn(session, packet)
            }
    }
}

object SessionPacketReceivedEventHandler {
    suspend fun handle(session: UranosSession, packet: IncomingPacket) {
        val handler = when (session.currentProtocol) {
            HandshakingProtocol -> HandshakingProtocolHandler
            StatusProtocol -> StatusProtocolHandler
            LoginProtocol -> LoginProtocolHandler
            PlayProtocol -> PlayProtocolHandler
            else -> return
        }

        handler.handle(session, packet)
    }
}
