package space.uranos.net

import io.netty.buffer.ByteBuf
import io.netty.channel.ChannelDuplexHandler
import io.netty.channel.ChannelHandlerContext
import io.netty.channel.ChannelPromise
import space.uranos.net.MinecraftProtocolDataTypes.writeVarInt
import space.uranos.net.packet.OutgoingPacket
import space.uranos.net.packet.OutgoingPacketCodec

class PacketMessageDuplexHandler(private val session: UranosSession) : ChannelDuplexHandler() {
    override fun channelActive(ctx: ChannelHandlerContext) {
        session.onConnect()
    }

    override fun channelInactive(ctx: ChannelHandlerContext) {
        session.onDisconnect(false)
    }

    override fun write(ctx: ChannelHandlerContext, msg: Any, promise: ChannelPromise) {
        val packet = msg as OutgoingPacket

        @Suppress("UNCHECKED_CAST")
        val codec =
            session.currentProtocol!!.getCodecByType(packet::class) as OutgoingPacketCodec<OutgoingPacket>

        val dst = ctx.alloc().buffer()
        dst.writeVarInt(codec.id)
        codec.encode(packet, dst)
        ctx.write(dst, promise)
    }

    override fun channelRead(ctx: ChannelHandlerContext, msg: Any) {
        session.packetsAdapter.onPacketReceived(msg as ByteBuf) // ByteBuf must be released by the session
    }

    override fun exceptionCaught(ctx: ChannelHandlerContext, cause: Throwable) {
        session.logger.error("An uncaught exception occurred", Exception(cause))
    }
}
