package space.uranos.net

import io.netty.bootstrap.ServerBootstrap
import io.netty.channel.ChannelOption
import io.netty.channel.epoll.Epoll
import io.netty.channel.epoll.EpollEventLoopGroup
import io.netty.channel.epoll.EpollServerSocketChannel
import io.netty.channel.kqueue.KQueue
import io.netty.channel.kqueue.KQueueEventLoopGroup
import io.netty.channel.kqueue.KQueueServerSocketChannel
import io.netty.channel.nio.NioEventLoopGroup
import io.netty.channel.socket.nio.NioServerSocketChannel
import space.uranos.UranosServer
import java.util.concurrent.CopyOnWriteArraySet

class UranosSocketServer(private val server: UranosServer) {
    private val bossGroup = createEventLoopGroup()
    private val workerGroup = createEventLoopGroup()
    private val bootstrap: ServerBootstrap = ServerBootstrap()
        .group(bossGroup, workerGroup)
        .channel(
            when {
                EPOLL_AVAILABLE -> EpollServerSocketChannel::class.java
                KQUEUE_AVAILABLE -> KQueueServerSocketChannel::class.java
                else -> NioServerSocketChannel::class.java
            }
        )
        .childOption(ChannelOption.SO_KEEPALIVE, true)
        .childOption(ChannelOption.TCP_NODELAY, true)
        .childHandler(UranosChannelInitializer(server))

    internal val sessions: MutableSet<UranosSession> = CopyOnWriteArraySet()

    fun bind() {
        bootstrap.bind(server.config.host, server.config.port)
    }

    companion object {
        private val EPOLL_AVAILABLE = Epoll.isAvailable()
        private val KQUEUE_AVAILABLE = KQueue.isAvailable()
        private fun createEventLoopGroup() = when {
            EPOLL_AVAILABLE -> EpollEventLoopGroup()
            KQUEUE_AVAILABLE -> KQueueEventLoopGroup()
            else -> NioEventLoopGroup()
        }
    }
}
