package space.uranos.net.packet.play

import kotlinx.coroutines.launch
import space.uranos.net.PacketReceivedEventHandler
import space.uranos.net.UranosSession
import space.uranos.player.UranosPlayer
import space.uranos.player.event.PlayerSettingsUpdateEvent

object ClientSettingsPacketHandler : PacketReceivedEventHandler<ClientSettingsPacket>() {
    override suspend fun handle(session: UranosSession, packet: ClientSettingsPacket) {
        val player = session.player

        if (player == null) {
            try {
                session.scope.launch { session.joinProcedure.onClientSettingsReceived(packet) }
            } catch (e: IllegalStateException) {
                session.failAndDisconnectBecauseOfClient("Client sent ClientSettingsPacket when it was not allowed")
            }
        } else {
            (player as UranosPlayer).settings =
                session.server.eventBus.emit(PlayerSettingsUpdateEvent(player, packet.asPlayerSettings())).settings
        }
    }
}
