package space.uranos.util

import io.netty.channel.ChannelFuture
import kotlinx.coroutines.suspendCancellableCoroutine
import okhttp3.Call
import okhttp3.Callback
import okhttp3.Response
import java.io.IOException
import kotlin.coroutines.Continuation
import kotlin.coroutines.suspendCoroutine

suspend fun ChannelFuture.awaitSuspending() {
    fun listen(c: Continuation<Unit>) {
        addListener { c.resumeWith(if (it.isSuccess) Result.success(Unit) else Result.failure(it.cause())) }
    }

    if (isCancellable) {
        suspendCancellableCoroutine<Unit> { c ->
            if (isCancelled) c.cancel()
            else {
                c.invokeOnCancellation { cancel(false) }
                listen(c)
            }
        }
    } else {
        suspendCoroutine<Unit> { c -> listen(c) }
    }
}

suspend fun Call.await(): Response = suspendCancellableCoroutine { c ->
    if (isCanceled()) c.cancel()
    c.invokeOnCancellation { cancel() }

    enqueue(object : Callback {
        override fun onResponse(call: Call, response: Response) {
            c.resumeWith(Result.success(response))
        }

        override fun onFailure(call: Call, e: IOException) {
            c.resumeWith(Result.failure(e))
        }
    })
}
