package space.uranos.util

import java.math.BigInteger
import java.security.*
import java.security.spec.X509EncodedKeySpec
import kotlin.random.Random

object EncryptionUtils {
    fun generateKeyPair(): KeyPair {
        try {
            val generator = KeyPairGenerator.getInstance("RSA")
            generator.initialize(1024)

            return generator.generateKeyPair()
        } catch (e: Exception) {
            throw Exception("Unable to generate RSA key pair", e)
        }
    }

    fun generateX509Key(base: Key): PublicKey {
        val encodedKeySpec = X509EncodedKeySpec(base.encoded)
        val keyFactory = KeyFactory.getInstance("RSA")

        return keyFactory.generatePublic(encodedKeySpec)
    }

    fun generateVerifyToken() = ByteArray(4).also { Random.Default.nextBytes(it) }

    fun toMinecraftStyleSha1String(value: ByteArray): String = BigInteger(value).toString(16)
}
